<?php

namespace EventPortal\Domain\Agenda\Models;

use EventPortal\Domain\Event\Models\Event;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\MorphToMany;

class Pathway extends Model
{
    public const KEY = 'pathway';
    public const DEFAULT = 'Default';

    /** @var array */
    protected $guarded = [];

    public static function generate(Event $event, string $name, string $description): Pathway
    {
        return Pathway::create([
            'event_id' => $event->id,
            'name' => $name,
            'description' => $description,
        ]);
    }

    public function sessions(): HasMany
    {
        return $this->hasMany(Session::class);
    }

    public function agenda(): MorphToMany
    {
        return $this->morphedByMany(Agenda::class, 'pathwayable');
    }

    public function scopeDefaultPathwayForEvent(Builder $query, Event $event): Builder
    {
        return $query->where('name', self::DEFAULT)->where('event_id', $event->id);
    }
}
