<?php

namespace EventPortal\Domain\Event\Form\Models;

use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Event\Form\Enums\FormType;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Model;
use EventPortal\Domain\Table\Concerns\HasTable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\MorphToMany;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;
use Spatie\Enum\Laravel\HasEnums;

class Form extends Model implements AuditableInterface
{
    use HasEnums, Auditable, HasTable;

    public const KEY = 'form';

    /** @var array<string,mixed> */
    protected $enums = [
        'type' => FormType::class.':nullable',
    ];

    /** @var array<string> */
    protected $with = [
        'steps.sections.questions.options',
        'steps.sections.questions.validations',
    ];

    /** @var array<string> */
    protected $fillable = [
        'name',
        'type',
        'uuid',
        'event_id',
    ];

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class);
    }

    public function steps(): HasMany
    {
        return $this->hasMany(FormStep::class)->orderBy('order');
    }

    public function pathways(): MorphToMany
    {
        return $this->morphToMany(Pathway::class, 'pathwayable');
    }

    public function scopeForEvent(Builder $query, Event $event): Builder
    {
        return $query->where('event_id', $event->id);
    }

    public static function typesAsOptions(): array
    {
        return collect(array_flip(FormType::toArray()))
            ->map(static function ($value): array {
                return Option::make($value)->toArray();
            })
            ->toArray();
    }
}
